/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import org.diacalc.mobile.components.HorizontalList;
import org.diacalc.mobile.components.ProductEditor;
import org.diacalc.mobile.components.AnimatedRenderer;
import org.diacalc.mobile.components.AnimatedCheckBoxRenderer;
import com.sun.lwuit.*;
import com.sun.lwuit.plaf.UIManager;
import com.sun.lwuit.layouts.*;
import com.sun.lwuit.list.*;
import com.sun.lwuit.events.*;
import org.diacalc.mobile.products.*;
import javax.microedition.midlet.*;
import org.diacalc.mobile.maths.*;
import com.sun.lwuit.plaf.Border;
import com.sun.lwuit.impl.midp.VirtualKeyboard;


public class ProdsForm extends FormBase implements ActionListener{
    public String getName() {
        return "Продукты";
    }
    public String getIconName(){
        return "prods";
    }
    private FishEyeTickerRenderer prRenderer = new FishEyeTickerRenderer();
    private static final int N_GR_COMMAND = 10;
    private static final int E_GR_COMMAND = 11;
    private static final int D_GR_COMMAND = 12;
    private static final int N_PR_COMMAND = 13;
    private static final int E_PR_COMMAND = 14;
    private static final int D_PR_COMMAND = 15;
    private static final int YES_COMMAND = 16;
    private static final int NO_COMMAND = 17;
    private static final int SEARCH_COMMAND = 18;

    private static final Command newGrCommand = new Command("Новая группа", N_GR_COMMAND);
    private static final Command editGrCommand = new Command("Изм. группу", E_GR_COMMAND);
    private static final Command delGrCommand = new Command("Удал. группу", D_GR_COMMAND);

    private static final Command newPrCommand = new Command("Новый прод.", N_PR_COMMAND);
    private static final Command editPrCommand = new Command("Изм. прод.", E_PR_COMMAND);
    private static final Command delPrCommand = new Command("Удал. прод.", D_PR_COMMAND);
    private static final Command searchCommand = new Command("Поиск", SEARCH_COMMAND);

    private static final Command yesCommand = new Command("Да",YES_COMMAND);
    private static final Command noCommand = new Command("Нет",NO_COMMAND);

    //components
    private TextField filter;
    private HorizontalList groups;
    private DefaultListModel prModel;
    private FilterProxyListModel proxyModel;
    private List prodsL;
    private Container bottomPane;

    private DCJmobile master = null;

    private boolean searchMode;
    private boolean flagGrDelete;
    private boolean touchDev;

    public void cleanup(){
        filter = null;
        searchMode = false;
        flagGrDelete = false;
        bottomPane = null;
        groups = null;
        prModel = null;
        proxyModel = null;
        searchMode = false;
        flagGrDelete = false;
    }

    protected void execute(final MIDlet m, boolean refresh) {
        if (refresh){
            cleanup();
            master = (DCJmobile)m;
            touchDev = Display.getInstance().isTouchScreenDevice();

            form.setLayout(new BorderLayout());

            groups = new HorizontalList(new DefaultListModel( master.getGroups() ));
            groups.setFocusable(false);

            prModel = new DefaultListModel( master.getProds() );

            proxyModel = new FilterProxyListModel(prModel);
            prodsL = new List(proxyModel){
                public void keyReleased(int keyC){
                    int ac = Display.getInstance().getGameAction(keyC);
                    if (keyC == new Integer('*').intValue()){
                        master.showForm(DCJmobile.MENU, false);
                    }
                    else if (ac!=Display.GAME_DOWN && ac!=Display.GAME_FIRE &&
                            ac!=Display.GAME_LEFT && ac!=Display.GAME_RIGHT &&
                            ac!=Display.GAME_UP){
                        if (filter==null){//нет никакого фильтра еще
                            createFilter();
                        }

                        if (!form.contains(filter) && !form.contains(bottomPane)){
                            if (touchDev){
                                form.addComponent(BorderLayout.SOUTH, bottomPane);
                            }else{
                                form.addComponent(BorderLayout.SOUTH, filter);
                            }

                            form.revalidate();
                        }

                        filter.setFocusable(true);
                        filter.keyReleased(keyC);
                        filter.setCursorPosition(filter.getText().length());
                        filter.requestFocus();
                    }
                    else super.keyReleased(keyC);
                }
            };

            groups.setSelectedIndex(0);

            proxyModel.filter("",((ProductGroup)groups.getSelectedItem()).getId());

            groups.addSelectionListener(new SelectionListener(){
                public void selectionChanged(int o,int n){
                    if (o!=n || flagGrDelete){
                        hideFilter();
                        proxyModel.filter("",
                               ((ProductGroup)groups.getSelectedItem()).getId());
                        if (prodsL.getModel().getSize()>0) prodsL.setSelectedIndex(0, true);
                        if (flagGrDelete) flagGrDelete = false;
                    }
                }

            });

            prodsL.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    ProductInBase prod = (ProductInBase)prodsL.getSelectedItem();
                    //вот тут если включен режим поиска нужно оформить выход из него
                    if (searchMode) searching();
                    else if (prod!=null){
                        //changemenu = true;
                        prod.setSelected(!prod.isSelected());
                        if (prod.isSelected()){
                            master.addProductToMenu(prod);
                        }
                        else{
                            master.removeProductFromMenu(prod);
                        }
                    }
                }
            });
            prodsL.setFireOnClick(false);

            form.addGameKeyListener(Display.GAME_LEFT,new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    if (!searchMode){
                        int idx = groups.getSelectedIndex();
                        if (idx>0 )                 groups.setSelectedIndex(idx-1);
                        else                        groups.setSelectedIndex(groups.size()-1);
                    }
                }
            } );
            form.addGameKeyListener(Display.GAME_RIGHT,new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    if (!searchMode){
                        int idx = groups.getSelectedIndex();
                        if (idx<(groups.size()-1) ) groups.setSelectedIndex(idx+1);
                        else                        groups.setSelectedIndex(0);
                    }
                }
            } );


            form.getContentPane().addComponent(BorderLayout.NORTH,groups);
            form.getContentPane().addComponent(BorderLayout.CENTER,prodsL);

            prodsL.setFixedSelection(List.FIXED_NONE_CYCLIC);//двигаемся по кругу "карусель"

            prodsL.setPaintFocusBehindList(true);
            prodsL.setListCellRenderer(prRenderer);
            prodsL.addSelectionListener(new SelectionListener(){
                public void selectionChanged(int o,int n){
                    if (o!=n){
                        prRenderer.restartTicker();
                    }
                }
             });

            form.setScrollable(false);

            form.addCommand(searchCommand);
            form.addCommand(delGrCommand);
            form.addCommand(editGrCommand);
            form.addCommand(newGrCommand);

            form.addCommand(delPrCommand);
            form.addCommand(editPrCommand);
            form.addCommand(newPrCommand);

            form.addCommandListener(ProdsForm.this);

            if (touchDev){
                bottomPane = new Container(new BorderLayout());
                Button btn2menu = new Button(" * ");
                btn2menu.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ev){
                        master.showForm(DCJmobile.MENU, false);
                    }
                });
                btn2menu.setFocusable(false);

                createFilter();

                bottomPane.addComponent(BorderLayout.CENTER, filter);
                bottomPane.addComponent(BorderLayout.EAST, btn2menu);
                form.addComponent(BorderLayout.SOUTH,bottomPane);
            }
        }
    }
    
    private void createFilter(){
        filter = new TextField();
        filter.addDataChangeListener(new DataChangedListener() {
            public void dataChanged(int type, int index) {
                proxyModel.filter(filter.getText(),
                    ((ProductGroup)groups.getSelectedItem()).getId());
            }
        });

        filter.addFocusListener(new FocusListener(){
            public void focusGained(Component c){}
            public void focusLost(Component c){
                if (touchDev) return;
                if (filter.getText().length()==0){
                    proxyModel.filter("",
                        ((ProductGroup)groups.getSelectedItem()).getId());
                    hideFilter();
                }
                else{
                    filter.setFocusable(false);
                }
            }
        });

        filter.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev){
                if (filter.getText().length()==0){
                    proxyModel.filter("",
                        ((ProductGroup)groups.getSelectedItem()).getId());
                    hideFilter();
                }
                else{
                    filter.setFocusable(false);
                }
            }
        });
        if (touchDev){
            VirtualKeyboard vk = DCJmobile.createRusTxtKeyboard();
            VirtualKeyboard.bindVirtualKeyboard(filter, vk);
        }
    }
    
    private void hideFilter(){//тут скрываем фильтр и/или меняем его на маленькую кнопку
        if (filter==null && bottomPane==null) return;//Первый запуск
        if (!form.contains(filter) && !form.contains(bottomPane)) return;

        filter.setText("");

        if (!touchDev){
            form.removeComponent(filter);
            form.revalidate();
        }

        prodsL.requestFocus();
        proxyModel.filter("",
                    ((ProductGroup)groups.getSelectedItem()).getId());
    }
    
    public void actionPerformed(ActionEvent ev){
        switch (ev.getCommand().getId()) {
            case N_GR_COMMAND: addGroup(); break;
            case E_GR_COMMAND: editGroup(); break;
            case D_GR_COMMAND: deleteGroup(); break;
            case N_PR_COMMAND: addProduct(); break;
            case D_PR_COMMAND: deleteProduct(); break;
            case E_PR_COMMAND: editProduct(); break;
            case SEARCH_COMMAND: searching(); break;
        }
    }
    private void searching(){
        if (searchMode){
            //Отменяем режим поиска
            //Нужно получить продукт, найтие его группу, и выделить его потом в списке
            ProductInBase pr = (ProductInBase)prodsL.getSelectedItem();

            groups.setSearch(false);//Сначала отключили в гор. списке
            hideFilter();
            searchMode = false;

            int i=0;
            for(;i<master.getGroups().size();i++){
                if ( ((ProductGroup)master.getGroups().elementAt(i)).getId()==
                        pr.getOwner()){
                    break;
                }
            }
            if (i<master.getGroups().size()){
                groups.setSelectedIndex(i);
                for(int j=0;j<prodsL.size();j++){
                    if ( ((ProductInBase)prodsL.getModel().getItemAt(j)).getId()==pr.getId() ){
                        prodsL.setSelectedIndex(j, true);
                        break;
                    }
                }
            }
        }else{
            //Включаем поиск
            groups.setSearch(true);
            searchMode = true;
            
            if (filter==null){//Это может случиться только если устройство без тачскрина
                createFilter();
                form.addComponent(BorderLayout.SOUTH, filter);
                form.revalidate();
            }
            String st = filter.getText();
            filter.setFocusable(true);
            filter.requestFocus();
            proxyModel.filter(st,-1);
            if (prodsL.size()>0) prodsL.setSelectedIndex(0, true);
        }
        
    }
    private void deleteProduct(){
        if (searchMode) return;
        if (prodsL.size()==0) return;
        ProductInBase p = (ProductInBase)prodsL.getSelectedItem();
        TextArea txt = new TextArea("Удалить продукт:\n" +
                p.getName()+"?");
        txt.setEditable(false);
        if (Dialog.show("Удал. продукт",
                txt,
                new Command[]{yesCommand,noCommand} ).getId()==YES_COMMAND){
            master.getProds().removeElement(p);
            String f = "";
            if (filter!=null){
                f = filter.getText();
            }
            proxyModel.filter(f,
                         p.getOwner() );
            master.setNeedSaveBase();
        }
    }
    private void editProduct(){
        if (searchMode) return;
        if (prodsL.size()==0) return;
        if (prodsL.getSelectedIndex()<0) return;
        ProductInBase old = (ProductInBase)prodsL.getSelectedItem();
        ProductEditor c = new ProductEditor( old ,master);
        
        int res = Dialog.show("Изм. продукт",
                c,
                new Command[]{yesCommand,noCommand} ).getId();

        if (res==YES_COMMAND){
            ProductInBase pr = c.getProduct();
            pr.setWeight(100f);
            if (pr.getName().length()>0){//Начинаем изменять
                old.setName(pr.getName());
                old.setProt(pr.getProt());
                old.setFat(pr.getFat());
                old.setCarb(pr.getCarb());
                old.setGi(pr.getGi());
                old.setMobile(true);
                old.setOwner(pr.getOwner());
                
                Ordering.setSortField(Ordering.NAME_FIELD);
                Ordering.setSortType(Ordering.SORT_BY_ASC);
                Ordering.sort(master.getProds());

                int pos = -1;
                for(int i=0;i<master.getGroups().size();i++){
                    if ( ((ProductGroup)master.getGroups().elementAt(i)).getId() ==
                            pr.getOwner() ){
                        pos = i;
                        break;
                    }
                }
                groups.setSelectedIndex(pos);
                hideFilter();

                proxyModel.filter("",
                         pr.getOwner() );
                pos = -1;
                for(int i=0;i<proxyModel.getSize();i++){
                    if ( ((ProductInBase)proxyModel.getItemAt(i)).getId()==old.getId() ){
                        pos = i;
                        break;
                    }
                }
                if (pos>=0){
                    prodsL.setSelectedIndex(pos, true);
                }
                master.setNeedSaveBase();
            }
        }
    }
    private void addProduct(){
        if (searchMode) return;
        ProductW prN = new ProductW();
        prN.setWeight(100f);
        ProductEditor c = new ProductEditor(new ProductInBase(prN,
                ((ProductGroup)groups.getModel().getItemAt(groups.getSelectedIndex())).getId()
                ),
                master);
        
        int res = Dialog.show("Добавить продукт",
                c,
                new Command[]{yesCommand,noCommand} ).getId();

        if (res==YES_COMMAND){
            ProductInBase pr = c.getProduct();
            pr.setWeight(100f);
            if (pr.getName().length()>0){
                prModel.addItem( pr );
                Ordering.setSortField(Ordering.NAME_FIELD);
                Ordering.sort(master.getProds());

                int pos = -1;
                for(int i=0;i<master.getGroups().size();i++){
                    if ( ((ProductGroup)master.getGroups().elementAt(i)).getId() ==
                            pr.getOwner() ){
                        pos = i;
                        break;
                    }
                }
                groups.setSelectedIndex(pos);
                hideFilter();

                proxyModel.filter("",
                         pr.getOwner() );
                pos = -1;
                for(int i=0;i<proxyModel.getSize();i++){
                    if ( ((ProductInBase)proxyModel.getItemAt(i)).getId()==pr.getId() ){
                        pos = i;
                        break;
                    }
                }
                if (pos>=0){
                    prodsL.setSelectedIndex(pos, true);
                }
                master.setNeedSaveBase();
            }
        }
    }
    private void editGroup(){
        if (searchMode) return;
        Container c = new Container(new BoxLayout(BoxLayout.Y_AXIS));
        Label lbl = new Label("Название группы");
        TextField fld = new TextField(
                ((ProductGroup)groups.getSelectedItem()).getName() );
        if (touchDev){
                VirtualKeyboard vk = DCJmobile.createRusTxtKeyboard();
                VirtualKeyboard.bindVirtualKeyboard(fld, vk);
        }
        c.addComponent(lbl);
        c.addComponent(fld);

        Command res = Dialog.show("Изм. группу", c,new Command[]{yesCommand,noCommand} );
        if (res.getId()==YES_COMMAND && fld.getText().length()>0){
            ((ProductGroup)groups.getSelectedItem()).setName(fld.getText());
            int ind = groups.getSelectedIndex();
            groups.setSelectedIndex(ind);
            master.setNeedSaveBase();
        }
    }
    private void addGroup(){
        if (searchMode) return;
        Container c = new Container(new BoxLayout(BoxLayout.Y_AXIS));
        Label lbl = new Label("Название группы");
        TextField fld = new TextField( "" );
        if (touchDev){
                VirtualKeyboard vk = DCJmobile.createRusTxtKeyboard();
                VirtualKeyboard.bindVirtualKeyboard(fld, vk);
        }
        c.addComponent(lbl);
        c.addComponent(fld);

        Command res = Dialog.show("Нов. группа", c,new Command[]{yesCommand,noCommand} );
        if (res.getId()==YES_COMMAND && fld.getText().length()>0){
            groups.addItem(new ProductGroup(fld.getText(),-1));
            groups.setSelectedIndex(groups.getModel().getSize()-1);
            proxyModel.filter("",
                         ((ProductGroup)groups.getSelectedItem()).getId());
            master.setNeedSaveBase();
        }
    }
    private void deleteGroup(){
        if (searchMode) return;
        if (groups.size()<2) return;
        hideFilter();

        TextArea txt = new TextArea("Будет удалена группа "
                + ((ProductGroup)groups.getSelectedItem()).getName() +
                " и все продукты в нее входящие");
        txt.setEditable(false);
        if (Dialog.show("Удл. группы",
                txt,
                new Command[]{yesCommand,noCommand} ).getId() == YES_COMMAND){
            //Удаляем
            int selO = groups.getSelectedIndex();
            
            int sel = 0;
            //тут надо сменить прокси модель заранее
            //если удаляем последний, то выбираем предыдущий, если первый, то выбираем следующий
            if (selO>0) sel = selO - 1;
            else sel = 1;

            int id = ((ProductGroup)groups.getSelectedItem()).getId();
            //удаляем все продукты из этой группы
            for(int i=master.getProds().size();i>0;i--){
                if ( ((ProductInBase)master.getProds().elementAt(i-1)).getOwner()==id ){
                    master.getProds().removeElementAt(i-1);
                }
            }

            groups.getModel().removeItem(groups.getSelectedIndex());

            //вот тут надо шагнуть назад или вперед
            flagGrDelete = true;
            if (selO>0) groups.setSelectedIndex(sel);//шаг назад
            else groups.setSelectedIndex(0);//т.к. удаляли последний, то возврат на шаг
            
            master.setNeedSaveBase();
        }

    }
    

    protected String getHelp() {
        return "В данном окне можно выбрать продукты для меню, для этого " +
                "необходимо отметить галкой нужный продукт.\n" +
                "Для перемещения по группам используйте джойстик вправо и влево, " +
                "либо на touch screen'ах нажимайте стрелки справа и слева от названия группы.\n" +
                "Для быстрой фильтрации продуктов начинайте набирать на клавиатуре " +
                "буквы, которые есть в составе названия искомого продукта.\n" +
                "Для быстрого перемещения в меню нажмите *";
    }

    /**
     * Demonstrates implementation of a renderer derived from a label
     * and the use of icons in the renderer
     */
    private static class FishEyeTickerRenderer extends CheckBox implements ListCellRenderer{
        private Label label = new Label();

      private AnimatedCheckBoxRenderer selectedRenderer = new AnimatedCheckBoxRenderer();
      private List parentList;
      private Container cont = new Container(new BoxLayout(BoxLayout.Y_AXIS));
      private AnimatedRenderer descr = new AnimatedRenderer(false);

      public FishEyeTickerRenderer() {
          super();
          setFocusable(false);
          label.getStyle().setBgTransparency(100);
          getStyle().setBgTransparency(0);

          selectedRenderer.setTickerEnabled(true);
          selectedRenderer.setFocus(true);
          selectedRenderer.getSelectedStyle().setBgTransparency(0);
          selectedRenderer.setFocusable(false);
          descr.setTickerEnabled(true);
          descr.getSelectedStyle().setBgTransparency(0);
          descr.setFocus(true);
          descr.setFocusable(true);

          cont.addComponent(selectedRenderer);
          cont.addComponent(descr);

          cont.getStyle().setBorder(Border.createLineBorder(1));
          cont.getStyle().setBgTransparency(255);
          cont.getStyle().setBgColor(0xFFFFFF);
      }
      public void restartTicker(){
          descr.resetPosition();
          selectedRenderer.resetPosition();
      }
      public boolean animate() {
          if(parentList != null && parentList.getComponentForm() != null) {
              if(selectedRenderer.isTickerRunning()) {
                  if(selectedRenderer.animate()) {
                      parentList.repaint();
                  }
              }
              if(descr.isTickerRunning()) {
                  if(descr.animate()) {
                      parentList.repaint();
                  }
              }
          }
          return super.animate();
      }

      public Component getListCellRendererComponent(List list, Object value, int index, boolean isSelected) {
          ProductInBase pr = (ProductInBase)value;
          if(isSelected) {
              selectedRenderer.setSelected(pr.isSelected());
              selectedRenderer.getListCellRendererComponent(list, pr.getName(), index, isSelected);
              descr.getListCellRendererComponent(list,
                        "Б:"+Str.round(pr.getProt(),1)+
                        " Ж:"+Str.round(pr.getFat(),1) +
                        " У:"+Str.round(pr.getCarb(),1) +
                        " ГИ:"+pr.getGi(),
                      index, isSelected);
                      //.setText();
              // sometimes the list asks for a dummy selected value for size calculations and this might
              // break the tickering state
              if(index == list.getSelectedIndex()) {
                  if(selectedRenderer.shouldTickerStart()) {
                      if(!selectedRenderer.isTickerRunning()) {
                          parentList = list;
                          list.getComponentForm().registerAnimated(this);
                          selectedRenderer.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                      }
                  } else {
                      if(selectedRenderer.isTickerRunning()) {
                          selectedRenderer.stopTicker();
                      }
                  }
                  if (descr.shouldTickerStart()){
                      if (!descr.isTickerRunning()){
                          parentList = list;
                          list.getComponentForm().registerAnimated(this);
                          descr.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                      }
                  }
                  else{
                      if(descr.isTickerRunning()) {
                          descr.stopTicker();
                      }
                  }
              }
              return cont;
          } else {
              setText(pr.getName());
              setSelected(pr.isSelected());
              return this;
          }
      }
      public Component getListFocusComponent(List list) {
        return label;
      }
  }
}
