/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */
package org.diacalc.mobile.components;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import com.sun.lwuit.list.*;
import com.sun.lwuit.plaf.UIManager;


public class AnimatedRenderer extends DefaultListCellRenderer {
    private static int SHIFT = 10;
    private int position = 0;
    private int delta = -2;//when neg move right, when pos move left
    private int ticksBeforeStart = 15;
    private int ticks = 0;
    private long delay = UIManager.getInstance().getLookAndFeel().getTickerSpeed();
    private long priorTick = -1;
    

    public AnimatedRenderer(boolean showNumbers){
        super(showNumbers);
        setFocusable(false);
        this.setTickerEnabled(true);
    }
    public void resetPosition() {
       position = 0;
       ticks = 0;
       delta = -2;
    }
    public void startTicker(long delay, boolean rightToLeft){
        super.startTicker(delay, rightToLeft);
        this.delay=delay;
        priorTick = System.currentTimeMillis();
    }
    public void incrementPosition() {
        if (priorTick<0) priorTick = System.currentTimeMillis();
        if ( (System.currentTimeMillis()-priorTick)<delay  ) return;
        int corr = (int) ((System.currentTimeMillis()-priorTick)/delay);
        priorTick = System.currentTimeMillis();
        if (ticks<ticksBeforeStart) ticks++; 
        else{
            if ( (delta<0 && -position>=( getSelectedStyle().getFont().stringWidth(getText())+ SHIFT -
                    getWidth() )) || (position>=0 && delta>0) ){
                ticks = 0;
                delta *= -1;
            }
            else{
                position += delta*corr;
                if (position>0) position = 0;
                else if (-position>=( getSelectedStyle().getFont().stringWidth(getText())+ SHIFT -
                    getWidth() )) position = getWidth() - getSelectedStyle().getFont().stringWidth(getText()) - SHIFT;

            }
        }
    }

    public boolean animate() {
        if (getWidth()<(getSelectedStyle().getFont().stringWidth(getText())+ SHIFT)){
            if (shouldTickerStart()){
                incrementPosition();
            }
            else{
                resetPosition();
            }
            setShiftText(position);
            return shouldTickerStart();
       }
       return false;
   }
}