/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile.maths;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import java.io.*;
import javax.microedition.io.*;


public class DoingPOST{
    private String answer = "";
    private String errorMsg = "";
    private boolean error = false;

    public DoingPOST(String pathToServer, String params){
        HttpConnection httpConn = null;
        InputStream is = null;
        OutputStreamWriter out = null;
        OutputStream os = null;

        try {
            // Open an HTTP Connection object
            httpConn = (HttpConnection)Connector.open(pathToServer,Connector.READ_WRITE);
            // Setup HTTP Request to POST
            httpConn.setRequestMethod(HttpConnection.POST);
            httpConn.setRequestProperty("User-Agent",
                "Profile/" + System.getProperty("microedition.configuration") +
                " Configuration/" + System.getProperty("microedition.profiles"));
            httpConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

            String locale = System.getProperty("microedition.locale");
            if (locale != null) {
                httpConn.setRequestProperty("Content-Language", locale);
            }
            httpConn.setRequestProperty("Content-Length", ""+params.getBytes().length );

            // --- write ---
            os = httpConn.openOutputStream();
            //out = new OutputStreamWriter(httpConn.openOutputStream(),"UTF-8");
            
            os.write(params.getBytes());
            //out.write(params);
            
            if (httpConn.getResponseCode()==HttpConnection.HTTP_OK){
                int length = (int)httpConn.getLength();
                is = httpConn.openInputStream();

                if (length==-1){//don't know the length
                    int chr;
                    ByteArrayOutputStream bout = new ByteArrayOutputStream();

                    while ((chr = is.read()) != -1)
                        bout.write(chr);

                    if (bout!=null){
                        answer = new String(bout.toByteArray(),"UTF-8");
                        bout.close();
                    }
                }
                else{//known length
                    DataInputStream dis = new DataInputStream(is);
                    byte[] data = new byte[length];
                    //try to read all the bytes returned from the server.
                    dis.readFully(data);
                    answer = new String(data,"UTF-8");
                    dis.close();
                }
            }
            else{
                errorMsg += "Server returned unhandled " +
                "response code. "+httpConn.getResponseCode();
                error = true;
            }

        }catch (Throwable t){
            errorMsg += "Exception while connecting to the internet:"+t.getMessage();
            error = true;
        }
        //Since only limited number of network objects can be in open state
        //it is necessary to clean them up as soon as we are done with them.
        finally {//httpConn out is
            if (is!=null)
                try{
                    is.close();
                }catch(Throwable t){
                    errorMsg += "Exception while close IS:"+t.getMessage();
                    error = true;
                }
            if(os != null)
                try{
                    os.close();
                }catch(Throwable t){
                    errorMsg += "Exeption while close OS:"+t.getMessage();
                    error = true;
                }
            if(httpConn != null)
                try{
                    httpConn.close();
                }catch(Throwable t){
                    errorMsg += "Exception while close HTTPCONN:"+t.getMessage();
                    error = true;
                }
        }
    }
    
    public String getAnswer(){
        return answer;
    }
    public boolean isError(){
        return error;
    }
    public String getErrorMessage(){
        return errorMsg;
    }
}